@@ -10,7 +10,7 @@ typedef void (t_fun0)(void);
 static AEffectDispatcherProc dispatch = 0;
 static int curProgram = 0;
 
-static float parameters[3];
+static float parameters[41];
 static ERect editorBounds = {0, 0, 320, 240};
 
 static char chunk[] = "This is the chunk for the FstPlugin.";
@@ -21,6 +21,8 @@ static VstMidiEvent s_midi;
 static unsigned char s_sysexDump[] = {0xF0, 0x01, 0x02, 0x03, 0x04, 0x03, 0x02, 0x01, 0xF7};
 static unsigned char s_midiDump[] = {0x80, 0x40, 0x0, 0};
 
+#define SIZEOF(x) (sizeof(x)/sizeof(*(x)))
+
 void crash() {
   t_fun0*f=0;
   /* crash */
@@ -29,7 +31,7 @@ void crash() {
   f();
 }
 
-void print_struct7(AEffect* effect) {
+void post_struct7(AEffect* effect) {
 #if 0
   auto *str = (double*)dispatch(effect, 7, 0, 65024, 0, 0.);
   for(size_t i=0; i<96/sizeof(*str); i++)
@@ -37,7 +39,7 @@ void print_struct7(AEffect* effect) {
   std::cout << std::endl;
 #else
   auto *vti = (VstTimeInfo*)dispatch(effect, 7, 0, 65024, 0, 0.);
-  print_timeinfo(vti);
+  post_timeinfo(vti);
 #endif
 }
 
@@ -55,35 +57,49 @@ t_fstPtrInt dispatch_v (AEffect* effect, int opcode, int index, t_fstPtrInt ival
   if(effect) {
     if(doprint) {
       char opcodestr[256];
-      printf("FstPlugin::plugin2host(%p, %s, %d, %lu, %p, %f) => ",  effect, hostCode2string(opcode, opcodestr, 255), index, ivalue, ptr, fvalue);
+      printf("FST::FstPlugin::plugin2host(%p, %s, %d, %lu, %p, %f) => ",  effect, hostCode2string(opcode, opcodestr, 255), index, ivalue, ptr, fvalue);
     }
     t_fstPtrInt result = dispatch(effect, opcode, index, ivalue, ptr, fvalue);
     if(doprint)
-      printf("FstPlugin::plugin2host: %lu (0x%lX)\n", result, result);
+      post("FstPlugin::plugin2host: %lu (0x%lX)", result, result);
     return result;
   }
   return 0xBEEF;
 }
 
-void test_opcodes(AEffect*effect, size_t toopcode = 100, size_t fromopcode=0) {
-  printf("testing host's dispatcher\n");
+void test_hostcodes(AEffect*effect, size_t toopcode = 100, size_t fromopcode=0, int skip_known=1) {
+  post("testing host's dispatcher");
   for(size_t opcode=fromopcode; opcode<toopcode; opcode++) {
     char buf[1024] = {0};
+    char opcodestr[256] = {0};
     snprintf(buf, 1023, "%s", "fudelDudelDa");
-    dispatch_v(effect, opcode, 0, 0, buf, 0.f);
+    if(skip_known && hostKnown(opcode)) {
+      if(opcode != audioMasterGetDirectory) {
+	post("skipping %s", hostCode2string(opcode, opcodestr, 255));
+	continue;
+      }
+    }
+    t_fstPtrInt result = dispatch_v(effect, opcode, 0, 0, buf, 0.f);
 #if 0
     if(*buf) {
-      printf("\t'%.*s'\n", 1024, buf);
+      post("\tBUFFER: '%.*s'", 1024, buf);
+    }
+#endif
+#if 1
+    if(result>0xFFFF) {
+      const char*res = (const char*)result;
+      post("\tRESULT: '%.*s'", 1024, res);
+      post_hex((void*)result, 64);
     }
 #endif
   }
-  printf("tested hosts's dispatcher with opcodes %d..%d\n", fromopcode, toopcode);
+  post("tested hosts's dispatcher with opcodes %d..%d", fromopcode, toopcode);
 }
 
 static void test_gettime_(AEffect*eff, t_fstInt32 flags) {
-  printf("getTime -> 0x%04X\n", flags);
+  post("getTime -> 0x%04X", flags);
   VstTimeInfo*t=(VstTimeInfo*)dispatch(eff, audioMasterGetTime, 0, flags, 0, 0.f);
-  print_timeinfo(t);
+  post_timeinfo(t);
 
 }
 
@@ -108,6 +124,10 @@ static int test_opcode35(AEffect*eff) {
   return kPlugSurroundFx;
 }
 
+static int test_opcode52(AEffect*eff) {
+  return 4321;
+}
+
 static int test_opcode70(AEffect*eff, char*ptr) {
   static int count = 0;
   count++;
@@ -118,20 +138,27 @@ static int test_opcode70(AEffect*eff, char*ptr) {
   return 0;
 }
 
-static int test_opcode66(AEffect*eff,
+static int test_GetMidiKeyName(AEffect*eff,
     t_fstInt32 opcode, int index,
     t_fstPtrInt ivalue, void* const ptr, float fvalue) {
-  /* this opcodes set the names of they keys of the virtual MIDI-keyboard in REAPER... */
-  //printf("opcode:%d\n", opcode);
+  /* this opcode sets the names of they keys of the virtual MIDI-keyboard in REAPER... */
+  //post("opcode:%d", opcode);
   //print_numbers((int*)ptr, 2);
-  char*str=(char*)ptr + 8;
-  int*iptr=(int*)ptr;
-  //printf("KEY: %d\t%d\n", iptr[0], iptr[1]);
-  snprintf(str, 16, "key[%d]%d=", iptr[0], iptr[1]);
-  for(size_t i=0; i<32; i++) {
-    //str[i+8]=i+64;
+  MidiKeyName*keyName = (MidiKeyName*)ptr;
+  /* 128 ... 512 */
+  /* valgrind shows errors at 256,192 */
+  char str[kVstMaxNameLen];
+  for(size_t i=0; i<sizeof(str); i++) {
+    str[i] = 65 + (i%26);
   }
-  return 0;
+  snprintf(str, sizeof(str)-1, "key[%d]%d=", keyName->thisProgramIndex, keyName->thisKeyNumber);
+  for(size_t i=0; i<sizeof(str); i++) {
+    if (0==str[i])
+      str[i] = '_';
+  }
+  str[sizeof(str)-1] = 0;
+  memcpy(keyName->keyName, str, sizeof(str));
+  return 1;
 }
 static int test_opcode62(AEffect*eff,
     t_fstInt32 opcode, int index,
@@ -147,8 +174,8 @@ static int test_opcode62(AEffect*eff,
   if(!retval)
     ret = cptr[0x44]+1;
 #endif
-  printf("OPCODE62: %d = %d -> %d\n", iptr[17], cptr[68], ret);
-  //print_hex(ptr, 256);
+  post("OPCODE62: %d = %d -> %d", iptr[17], cptr[68], ret);
+  //post_hex(ptr, 256);
   //snprintf((char*)ptr, 128, "OPCODE62");
   //*ccptr = (char*)"OPCODE62";
   /* cptr[4] must be "true" in order to keep running */
@@ -161,8 +188,8 @@ static int test_opcode56(AEffect*eff,
     t_fstPtrInt ivalue, void* const ptr, float fvalue) {
   int i;
   char*cptr = (char*)ptr;
-  printf("OPCODE56\n");fflush(stdout);
-  print_hex(ptr, 160);
+  post("OPCODE56");
+  post_hex(ptr, 160);
   snprintf(cptr, 16, "OPCODE56");
 #if 1
   for(i=0; i<0x99; i++) {
@@ -178,8 +205,9 @@ static void test_hostVendorSpecific(AEffect*eff) {
   int listadj[2] = { 1, 3 };
   int opcode;
   //audioMasterCallback(audioMasterVendorSpecific, 0xdeadbeef, audioMasterAutomate, listadj, 0.0);
-  for(opcode=0; opcode<100; opcode++) {
+  for(opcode=0; opcode<128; opcode++) {
     int skip = 0;
+    char buf[1024];
     if(hostKnown(opcode)) continue;
     switch (opcode) {
     case 11: /* returns 3 */
@@ -192,9 +220,14 @@ static void test_hostVendorSpecific(AEffect*eff) {
     case 53:
       skip = 1;
     }
-    if(skip)continue;
-    printf("testing %d\n", opcode); fflush(stdout);
-    dispatch_v(eff, opcode, 0xDEADBEEF, audioMasterAutomate, listadj, 0.0);
+    //if(skip)continue;
+    memset(buf, 0, sizeof(buf));
+    sprintf(buf, "%s", "hello");
+    post("testing %d", opcode);
+    //dispatch_v(eff, opcode, 0xDEADBEEF, audioMasterAutomate, listadj, 0.0);
+    dispatch_v(eff, opcode, 0, 0, buf, 0.0);
+    if(*buf)
+      post("\ttest: %s", buf);
   }
 }
 
@@ -202,7 +235,7 @@ static int test_hostUpdateDisplay(AEffect*eff) {
   int opcode = audioMasterUpdateDisplay;
   int res;
   res =  dispatch_v(eff, opcode, 0, 0, NULL, 0.0);
-  printf("%s: opcode(%d) -> %d\n", __FUNCTION__, opcode, res);fflush(stdout);
+  post("%s: opcode(%d) -> %d", __FUNCTION__, opcode, res);
   return res;
 }
 
@@ -210,8 +243,7 @@ static int test_opcode26(AEffect*eff,
                          t_fstInt32 opcode, int index,
                          t_fstPtrInt ivalue, void* const ptr, float fvalue) {
   int result = (index<2);
-  printf("OPCODE26[%d]: %d\n", index, result);
-  fflush(stdout);
+  post("OPCODE26[%d]: %d", index, result);
   return result;
 }
 
@@ -228,7 +260,7 @@ static void test_opcode25(AEffect*eff,
 
   return;
   char filename[128];
-  print_hex(ptr, 256);
+  post_hex(ptr, 256);
   static int count = 0;
   count++;
   sprintf(filename, "midi/dump.%d", count);
@@ -244,11 +276,10 @@ static void test_opcode42(AEffect*eff,
   /* ptr and ivalue point to VstSpeakerArrangement* */
   VstSpeakerArrangement*iarr = (VstSpeakerArrangement*)ivalue;
   VstSpeakerArrangement*oarr = (VstSpeakerArrangement*)ptr;
-  print_hex(iarr, 16);
-  print_hex(oarr, 16);
-  printf("JMZ| %d | 0x%X | %d |\n", iarr->type, iarr->type, iarr->numChannels);
-  printf("JMZ| %d | 0x%X | %d |\n", oarr->type, oarr->type, oarr->numChannels);
-  fflush(stdout);
+  post_hex(iarr, 16);
+  post_hex(oarr, 16);
+  post("JMZ| %d | 0x%X | %d |", iarr->type, iarr->type, iarr->numChannels);
+  post("JMZ| %d | 0x%X | %d |", oarr->type, oarr->type, oarr->numChannels);
 #ifdef NUM_INPUTS
   crash();
 #endif
@@ -262,8 +293,8 @@ static bool dispatcher_printEff(AEffect*eff,
     t_fstInt32 opcode, int index,
     t_fstPtrInt ivalue, void* const ptr, float fvalue) {
   char opcodestr[512];
-  printf("FstPlugin::dispatch(%s, %d, %ld, %p, %f);\n",
-         effCode2string(opcode, opcodestr, 512), index, ivalue, ptr, fvalue);
+  post("FstPlugin::dispatch(%s, %s, %s, %p, %f);",
+       effCode2string(opcode, opcodestr, 512), int2str(index), int2str2(ivalue), ptr, fvalue);
   return true;
 }
 static bool dispatcher_skip(t_fstInt32 opcode) {
@@ -271,11 +302,14 @@ static bool dispatcher_skip(t_fstInt32 opcode) {
   case effEditIdle:
     /* called permanently, if GUI is visible */
     return true;
-  case 53:
+  case effIdle /* 53 */:
     /* REAPER calls this permanently */
-    //printf("53...\n");
-    //print_struct7(eff);
-    return true;
+    //post("53...");
+    //post_struct7(eff);
+
+    //return true;
+  default:
+    break;
   }
   return false;
 }
@@ -286,13 +320,14 @@ static bool dispatcher_noprint(t_fstInt32 opcode) {
   case effGetParamName:
   case effProcessEvents:
   case effVendorSpecific:
+  case effIdle:
   case effEditIdle:
-  case fst_effGetMidiNoteName:
+  case effGetMidiKeyName:
     return true;
   }
   return false;
 }
-static void print_ptr4opcode(t_fstInt32 opcode, void*const ptr) {
+static void post_ptr4opcode(t_fstInt32 opcode, void*const ptr) {
   if(!ptr)return;
   char*str = (char*)ptr;
   switch(opcode) {
@@ -304,26 +339,56 @@ static void print_ptr4opcode(t_fstInt32 opcode, void*const ptr) {
     return;
   }
   if(str && *str)
-    printf("\tFtsClient::dispatcher(ptr='%.*s')\n", 64, str);
-  //if(str)print_hex(str, 96);
+    post("\tFstClient::dispatcher(ptr='%.*s')", 64, str);
+  //if(str)post_hex(str, 96);
 }
 static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPtrInt ivalue, void* const ptr, float fvalue) {
-  if(25==opcode)      test_processLevel(eff);
-  //if(53==opcode)    test_gettime(eff);
-  if(26==opcode)      return test_opcode26(eff, opcode, index, ivalue, ptr, fvalue);
+  switch(opcode) {
+  case effProcessEvents: /* 25 */
+    test_processLevel(eff);
+    break;
+  case effIdle: /* 53 */
+    //test_gettime(eff);
+    break;
+#if 0
+  case effCanBeAutomated: /* 26 */
+    return test_opcode26(eff, opcode, index, ivalue, ptr, fvalue);
+#endif
+  default:
+    break;
+  }
   if(dispatcher_skip(opcode))return 0;
   if(!dispatcher_noprint(opcode)) {
     dispatcher_printEff(eff, opcode, index, ivalue, ptr, fvalue);
-    print_ptr4opcode(opcode, ptr);
+    post_ptr4opcode(opcode, ptr);
   }
 
   switch(opcode) {
   default: break;
     //case effGetVstVersion: return 2400;
-  case 26:
+  case effOpen:
+    test_hostcodes(eff, 128, 0);
+    break;
+  case effClose:
+    test_hostcodes(eff, 128, 0);
+    break;
+  case effIdle: /* 53 */ {
+    static int idlecount = 0;
+    idlecount++;
+
+    if (100 == idlecount) {
+      post("idled for %d iterations", idlecount);
+      test_hostVendorSpecific(eff);
+    }
+  }
+    break;
+  case effCanBeAutomated: /* 26 */
     return test_opcode26(eff, opcode, index, ivalue, ptr, fvalue);
   case effGetPlugCategory:
     return test_opcode35(eff);
+  case effGetProgram:
+    test_hostcodes(eff, 128, 0);
+    break;
   case effShellGetNextPlugin:
     return test_opcode70(eff, (char*)ptr);
   case effGetVendorString:
@@ -337,6 +402,8 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
     test_opcode42(eff, opcode, index, ivalue, ptr, fvalue);
     return 0;
 #endif
+  case 52:
+    return test_opcode52(eff);
 #if 1
   case 56:
     return test_opcode56(eff, opcode, index, ivalue, ptr, fvalue);
@@ -347,8 +414,8 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
     return 1;
   case 62:
     return test_opcode62(eff, opcode, index, ivalue, ptr, fvalue);
-  case fst_effGetMidiNoteName:
-    return test_opcode66(eff, opcode, index, ivalue, ptr, fvalue);
+  case effGetMidiKeyName:
+    return test_GetMidiKeyName(eff, opcode, index, ivalue, ptr, fvalue);
   case effEditGetRect:
     *((ERect**)ptr) = &editorBounds;
     return (t_fstPtrInt)&editorBounds;
@@ -357,32 +424,33 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
       char**strptr=(char**)ptr;
       *strptr=chunk;
     }
-    //printf("getChunk: %d bytes @ %p\n", sizeof(chunk), chunk);
+    //post("getChunk: %d bytes @ %p", sizeof(chunk), chunk);
     return sizeof(chunk);
   case effSetProgram:
-    //printf("setting program to %d\n", ivalue);
+    //post("setting program to %d", ivalue);
     curProgram = ivalue;
     return 1;
   case effGetProgramName:
     snprintf((char*)ptr, 32, "FstProgram%d", curProgram);
-    //printf("JMZ:setting program-name to %s\n", (char*)ptr);
+    //post("JMZ:setting program-name to %s", (char*)ptr);
     return 1;
   case effGetParamLabel:
     snprintf((char*)ptr, 32, "°");
     return 0;
   case effGetParamName:
-    if(index>=sizeof(parameters))
-      index=sizeof(parameters);
-    snprintf((char*)ptr, 32, "rotation%c", index+88);
+    if(index>=SIZEOF(parameters))
+      index=SIZEOF(parameters);
+    snprintf((char*)ptr, 32, "rotation%c", 65+(index%26));
+    post("======================= effGetParamName[%d/%d] %s", index, SIZEOF(parameters), ptr);
     return 0;
   case effGetParamDisplay:
-    if(index>=sizeof(parameters))
-      index=sizeof(parameters);
+    if(index>=SIZEOF(parameters))
+      index=SIZEOF(parameters);
     snprintf((char*)ptr, 32, "%+03d", int((parameters[index]-0.5)*360+0.5));
     return 0;
   case effEditClose:
-    printf("EDIT Close!\n");
-    test_hostVendorSpecific(eff);
+    post("EDIT Close!");
+    //test_hostVendorSpecific(eff);
     return 1;
   case effCanDo: {
     typedef struct _cando {
@@ -390,26 +458,35 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
       unsigned int res;
     } t_cando;
     t_cando candos[] = {
+#if 1
       {"receiveVstEvents", 1},
+      {"receiveVstMidiEvent", 1},
       {"receiveVstMidiEvents", 1},
       {"sendVstEvents", 1},
+      {"sendVstMidiEvent", 1},
       {"sendVstMidiEvents", 1},
+#endif
+      {"bypass", 1},
       /* announcing 'wantsChannelCountNotifications' makes REAPER send out effSetSpeakerArrangement */
-      //{"wantsChannelCountNotifications", 1},
+      {"wantsChannelCountNotifications", 0},
+#if 0
       {"hasCockosExtensions", 0xbeef0000},
       {"hasCockosEmbeddedUI", 0xbeef0000},
       {"hasCockosNoScrollUI", 1 /* ? */ },
       {"hasCockosSampleAccurateAutomation", 1 /* ? */ },
       {"hasCockosViewAsConfig", 1 /* ? */ },
       {"cockosLoadingConfigAsParameters", 1 /* ? */ },
+#endif
       {0, 0} /* END */
     };
     t_cando*cando;
-    printf("canDo '%s'?\n", (char*)ptr);
+    post("canDo '%s'?", (char*)ptr);
     if(!ptr) return 0;
     for(cando=candos; cando->ID; cando++) {
-      if(!strcmp((char*)ptr, cando->ID))
+      if(!strcmp((char*)ptr, cando->ID)) {
+	post("canDo '%s': %d", cando->ID, cando->res);
         return cando->res;
+      }
     }
   }
     return 0;
@@ -419,13 +496,13 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
       do {
         static bool first=true;
         if(first) {
-          test_opcodes(eff, 50);
+          test_hostcodes(eff, 50);
         } else {
           auto *str = (char*)dispatch_v(eff, audioMasterGetTime, 0, 65024, 0, 0.);
           char filename[128];
           static int icount = 0;
           snprintf(filename, 127, "./testdump.%d", icount);
-          printf("OUTFILE[%d]: %s\n", icount, filename);
+          post("OUTFILE[%d]: %s", icount, filename);
           icount++;
           dump_data(filename, str, 512);
         }
@@ -436,11 +513,11 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
       break;
   case effVendorSpecific: {
     char buf[1024];
-    printf("FstPlugin::%s(effVendorSpecific/%s)\n", __FUNCTION__, effCode2string(index, buf, sizeof(buf)));
+    post("FstPlugin::%s(effVendorSpecific/%s)", __FUNCTION__, effCode2string(index, buf, sizeof(buf)));
     switch(index) {
       /* the 'stCA' code seems to be common among hosts ... */
     case FOURCC("stCA"):
-      printf("FstPlugin::%s(effVendorSpecific/%.4s/%.4s) ", __FUNCTION__, fourcc2str(index), fourcc2str(ivalue, buf));
+      printf("FST::FstPlugin::%s(effVendorSpecific/%.4s/%.4s) ", __FUNCTION__, fourcc2str(index), fourcc2str(ivalue, buf));
       switch(ivalue) {
       case FOURCC("Whee"):
         printf("MouseWheel");
@@ -449,7 +526,7 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
         printf("(0x%08X)", ivalue);
         break;
       }
-      printf("\n");
+      printf("\n");post();
       break;
       /*
         https://www.reaper.fm/sdk/vst/vst_ext.php
@@ -460,7 +537,7 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
 #if 0
           ((double *)ptr)[0] = double(-180.);
           ((double *)ptr)[1] = double( 180.);
-          printf("vendorspecific BEEF!\n");
+          post("vendorspecific BEEF!");
           return 0xbeef;
 #endif
         }
@@ -470,7 +547,7 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
         REAPER: override instance name
       */
       char**ccptr = (char**)ptr;
-      *ccptr = (char*)"OtherName";
+      *ccptr = (char*)"OtherName @" __DATE__ "/" __TIME__;
       return 0xF00D;
     }
       break;
@@ -487,8 +564,8 @@ static t_fstPtrInt dispatcher(AEffect*eff, t_fstInt32 opcode, int index, t_fstPt
   } /* case effVendorSpecific */
     break;
   }
-  //printf("FstPlugin::dispatch(%p, %d, %d, %d, %p, %f)\n", eff, opcode, index, ivalue, ptr, fvalue);
-  //printf("JMZ\n");
+  //post("FstPlugin::dispatch(%p, %d, %d, %d, %p, %f)", eff, opcode, index, ivalue, ptr, fvalue);
+  //post("JMZ");
 
   return 0;
 }
@@ -502,41 +579,41 @@ static void find_audioMasterSizeWindow() {
     int height = 640;
     if(37==opcode)
       continue;
-    printf("trying: %d\n", opcode);
+    post("trying: %d", opcode);
     t_fstPtrInt res = dispatch(0, opcode, width, height, 0, 0);
-    printf("%s[%dx%d] returned %ld\n", hostCode2string(opcode, hostcode, 512), width, height, res);
+    post("%s[%dx%d] returned %ld", hostCode2string(opcode, hostcode, 512), width, height, res);
   }
 }
 
 static void setParameter(AEffect*eff, int index, float value) {
-  //printf("FstPlugin::setParameter(%p)[%d] -> %f\n", eff, index, value);
-  if(index>=sizeof(parameters))
-    index=sizeof(parameters);
+  //post("FstPlugin::setParameter(%p)[%d] -> %f", eff, index, value);
+  if(index>=SIZEOF(parameters))
+    index=SIZEOF(parameters);
   parameters[index] = value;
 
 }
 static float getParameter(AEffect*eff, int index) {
-  if(index>=sizeof(parameters))
-    index=sizeof(parameters);
-  //printf("FstPlugin::getParameter(%p)[%d] <- %f\n", eff, index, parameters[index]);
+  if(index>=SIZEOF(parameters))
+    index=SIZEOF(parameters);
+  //post("FstPlugin::getParameter(%p)[%d] <- %f", eff, index, parameters[index]);
   return parameters[index];
 }
 static void process(AEffect*eff, float**indata, float**outdata, int sampleframes) {
 #if 0
-  printf("FstPlugin::process0(%p, %p, %p, %d) -> %f\n", eff, indata, outdata, sampleframes, indata[0][0]);
+  post("FstPlugin::process0(%p, %p, %p, %d) -> %f", eff, indata, outdata, sampleframes, indata[0][0]);
   test_gettime(eff);
 #endif
 }
 static void processReplacing(AEffect*eff, float**indata, float**outdata, int sampleframes) {
-#if 1
-  printf("FstPlugin::process1(%p, %p, %p, %d) -> %f\n", eff, indata, outdata, sampleframes, indata[0][0]);
+#if 0
+  post("FstPlugin::process1(%p, %p, %p, %d) -> %f", eff, indata, outdata, sampleframes, indata[0][0]);
   test_processLevel(eff);
   //test_gettime(eff);
 #endif
 }
 static void processDoubleReplacing(AEffect*eff, double**indata, double**outdata, int sampleframes) {
 #if 0
-  printf("FstPlugin::process2(%p, %p, %p, %d) -> %g\n", eff, indata, outdata, sampleframes, indata[0][0]);
+  post("FstPlugin::process2(%p, %p, %p, %d) -> %g", eff, indata, outdata, sampleframes, indata[0][0]);
   test_gettime(eff);
 #endif
 }
@@ -544,8 +621,8 @@ static void processDoubleReplacing(AEffect*eff, double**indata, double**outdata,
 extern "C"
 AEffect*VSTPluginMain(AEffectDispatcherProc dispatch4host) {
   dispatch = dispatch4host;
-  printf("FstPlugin::main(%p)\n", dispatch4host);
-  for(size_t i=0; i<sizeof(parameters); i++)
+  post("FstPlugin::main(%p)", dispatch4host);
+  for(size_t i=0; i<SIZEOF(parameters); i++)
     parameters[i] = 0.5;
 
   AEffect* eff = new AEffect;
@@ -557,13 +634,16 @@ AEffect*VSTPluginMain(AEffectDispatcherProc dispatch4host) {
   eff->setParameter = setParameter;
 
   eff->numPrograms = 1;
-  eff->numParams = 3;
+  eff->numParams = SIZEOF(parameters);
+  eff->numInputs  = 1;
+  eff->numOutputs = 1;
+
 #ifdef NUM_INPUTS
   eff->numInputs  = NUM_INPUTS;
-  eff->numOutputs = NUM_INPUTS;
-#else
-  eff->numInputs  = 6;
-  eff->numOutputs = 6;
+  eff->numOutputs = NUM_INPUTS+1;
+#endif
+#ifdef NUM_OUTPUTS
+  eff->numOutputs = NUM_OUTPUTS;
 #endif
   eff->float1 = 1.;
   eff->object = eff;
@@ -573,37 +653,39 @@ AEffect*VSTPluginMain(AEffectDispatcherProc dispatch4host) {
   //eff->flags |= effFlagsProgramChunks;
   eff->flags |= effFlagsCanReplacing;
   eff->flags |= effFlagsCanDoubleReplacing;
-
+#if 0
   eff->flags |= effFlagsHasEditor;
-
+#endif
   eff->processReplacing = processReplacing;
   eff->processDoubleReplacing = processDoubleReplacing;
-  print_aeffect(eff);
+  post_aeffect(eff);
 
-  const char* canDos[] = { "supplyIdle",
-                           "sendVstEvents",
-                           "sendVstMidiEvent",
-                           "sendVstTimeInfo",
-                           "receiveVstEvents",
-                           "receiveVstMidiEvent",
-                           "supportShell",
-                           "sizeWindow",
-                           "shellCategory" };
-  for(size_t i = 0; i<(sizeof(canDos)/sizeof(*canDos)); i++) {
+  const char* canDos[] = {
+    "supplyIdle",
+    "sendVstEvents",
+    "sendVstMidiEvent",
+    "sendVstTimeInfo",
+    "receiveVstEvents",
+    "receiveVstMidiEvent",
+    "supportShell",
+    "sizeWindow",
+    "shellCategory"
+  };
+  for(size_t i = 0; i<(SIZEOF(canDos)); i++) {
     char buf[512] = {0};
     char hostcode[512] = {0};
     snprintf(buf, 511, canDos[i]);
     buf[511]=0;
     t_fstPtrInt res = dispatch(0, audioMasterCanDo, 0, 0, buf, 0);
     if(*buf)
-      printf("%s['%.*s'] returned %ld\n", hostCode2string(audioMasterCanDo, hostcode, 512), 512, buf, res);
+      post("%s['%.*s'] returned %ld", hostCode2string(audioMasterCanDo, hostcode, 512), 512, buf, res);
   }
 
   //find_audioMasterSizeWindow();
 
   char buf[512] = {0};
   dispatch(eff, audioMasterGetProductString, 0, 0, buf, 0.f);
-  printf("masterProduct: %s\n", buf);
+  post("masterProduct: '%s'", buf);
 
   s_ves = (VstEvents*)calloc(1, sizeof(VstEvents)+2*sizeof(VstEvent*));
   s_ves->numEvents = 2;
@@ -625,9 +707,10 @@ AEffect*VSTPluginMain(AEffectDispatcherProc dispatch4host) {
   s_sysex.dumpBytes = sizeof(s_sysexDump);
   s_sysex.sysexDump = (char*)s_sysexDump;
 
-  print_events(s_ves);
+  post_events(s_ves);
 
-  printf("=====================================\n\n\n");
+  post("=====================================");
+  post(""); post("");
 
   return eff;
 }
