@@ -2,6 +2,14 @@
 #define FST_FST_UTILS_H_
 
 #include "fst.h"
+#include <stdarg.h>
+
+#ifdef _MSC_VER
+# define PERTHREAD __declspec(thread)
+#else
+# define PERTHREAD __thread
+#endif /* _MSC_VER */
+
 
 #ifdef _WIN32
 # include <windows.h>
@@ -19,25 +27,58 @@ static void fstpause(float duration=1.0) {
 
 #define FOURCC(fourcc) ((fourcc[0]<<24) | (fourcc[1]<<16) | (fourcc[2]<< 8) | (fourcc[3]<< 0))
 
-static char*fourcc2str(unsigned int fourcc, char buf[4]) {
+static const char*fourcc2str(unsigned int fourcc, char buf[4]) {
   buf[0] = (fourcc >> 24) & 0xFF;
   buf[1] = (fourcc >> 16) & 0xFF;
   buf[2] = (fourcc >>  8) & 0xFF;
   buf[3] = (fourcc >>  0) & 0xFF;
   return buf;
 }
-static char*fourcc2str(unsigned int fourcc) {
-  static char buf[5];
+static const char*fourcc2str(unsigned int fourcc) {
+  static PERTHREAD char buf[5];
   buf[4] = 0;
   return fourcc2str(fourcc, buf);
 }
 
-static void print_hex(void*ptr, size_t length) {
-  printf("DATA@%p [%d]", ptr, length);
+static const char*int2str(t_fstPtrInt ivalue, char*output, size_t length) {
+  if(ivalue>0xFFFFFFFF) {
+    snprintf(output, length, "0x%X", ivalue);
+  } else {
+    snprintf(output, length, "%llu=0x%X", ivalue, ivalue);
+  }
+  return output;
+}
+
+static const char*int2str(t_fstPtrInt ivalue) {
+  static PERTHREAD char buffer[1024];
+  return int2str(ivalue, buffer, 1024);
+}
+static const char*int2str2(t_fstPtrInt ivalue) {
+  static PERTHREAD char buffer[1024];
+  return int2str(ivalue, buffer, 1024);
+}
+
+int post(const char *format=0, ...) {
+   va_list args;
+   int result = 0;
+
+   if(format) {
+     result+=printf("FST::");
+     va_start(args, format);
+     result+=vprintf(format, args);
+     va_end(args);
+     printf("\n");
+   }
+   fflush(stdout);
+   return result;
+}
+
+static void post_hex(void*ptr, size_t length) {
+  printf("FST::DATA@%p [%d]", ptr, length);
   unsigned char* data = (unsigned char*)ptr;
   if(data) {
     for(size_t i=0; i<length; i++) {
-      if(!(i%16))printf("\n%04x\t", i);
+      if(!(i%16))printf("\nFST %04x\t", i);
       if(!(i% 8))printf(" ");
       printf(" %02X", *data++);
     }
@@ -74,7 +115,6 @@ static void print_numbers(type*data, size_t length) {
   std::cout << std::endl;
 }
 
-
 #define FST_UTILS__OPCODESTR(x)                   \
   case x:                                         \
   if(x>100000)                                    \
@@ -84,7 +124,7 @@ static void print_numbers(type*data, size_t length) {
   output[length-1] = 0;                           \
   return output
 
-static char*effCode2string(size_t opcode, char*output, size_t length) {
+static const char*effCode2string(size_t opcode, char*output, size_t length) {
   switch(opcode) {
     FST_UTILS__OPCODESTR(effCanBeAutomated);
     FST_UTILS__OPCODESTR(effCanDo);
@@ -101,7 +141,7 @@ static char*effCode2string(size_t opcode, char*output, size_t length) {
     FST_UTILS__OPCODESTR(effEditTop);
     FST_UTILS__OPCODESTR(effGetChunk);
     FST_UTILS__OPCODESTR(effGetCurrentMidiProgram);
-    FST_UTILS__OPCODESTR(fst_effGetMidiNoteName);
+    FST_UTILS__OPCODESTR(effGetMidiKeyName);
     FST_UTILS__OPCODESTR(effGetEffectName);
     FST_UTILS__OPCODESTR(effGetInputProperties);
     FST_UTILS__OPCODESTR(effGetNumMidiInputChannels);
@@ -142,10 +182,9 @@ static char*effCode2string(size_t opcode, char*output, size_t length) {
     FST_UTILS__OPCODESTR(effVendorSpecific);
   default: break;
   }
-  snprintf(output, length, "%llu=0x%X", opcode, opcode);
-  return output;
+  return(int2str(opcode, output, length));
 }
-static char*hostCode2string(t_fstPtrInt opcode, char*output, size_t length) {
+static const char*hostCode2string(t_fstPtrInt opcode, char*output, size_t length) {
   switch(opcode) {
     FST_UTILS__OPCODESTR(audioMasterAutomate);
     FST_UTILS__OPCODESTR(audioMasterVersion);
@@ -201,62 +240,67 @@ static int effKnown(t_fstPtrInt opcode) {
   if(opcode>=100000)
     return 0;
   switch(opcode) {
-    case effCanBeAutomated:
-    case effCanDo:
-    case effClose:
-    case effConnectInput:
-    case effConnectOutput:
-    case effEditClose:
-    case effEditDraw:
-    case effEditGetRect:
-    case effEditIdle:
-    case effEditMouse:
-    case effEditOpen:
-    case effEditSleep:
-    case effEditTop:
-    case effGetChunk:
-    case effGetCurrentMidiProgram:
-    case fst_effGetMidiNoteName:
-    case effGetEffectName:
-    case effGetInputProperties:
-    case effGetNumMidiInputChannels:
-    case effGetNumMidiOutputChannels:
-    case effGetOutputProperties:
-    case effGetParamDisplay:
-    case effGetParamLabel:
-    case effGetParamName:
-    case effGetPlugCategory:
-    case effGetProductString:
-    case effGetProgram:
-    case effGetProgramName:
-    case effGetProgramNameIndexed:
-    case effGetSpeakerArrangement:
-    case effGetTailSize:
-    case effGetVendorString:
-    case effGetVendorVersion:
-    case effGetVstVersion:
-    case effIdentify:
-    case effIdle:
-    case effKeysRequired:
-    case effMainsChanged:
-    case effOpen:
-    case effProcessEvents:
-    case effSetBlockSize:
-    case effSetBypass:
-    case effSetChunk:
-    case effSetProcessPrecision:
-    case effSetProgram:
-    case effSetProgramName:
-    case effSetSampleRate:
-    case effSetSpeakerArrangement:
-    case effSetTotalSampleToProcess:
-    case effShellGetNextPlugin:
-    case effStartProcess:
-    case effStopProcess:
-    case effString2Parameter:
-    case effVendorSpecific:
-      return 1;
   default: break;
+    /*
+      grep "^[[:space:]]*FST_EFFECT_OPCODE" fst/fst.h | sed -e 's|,.*||' -e 's|).*||' -e 's|$|:|' -e 's|.*(|  case eff|' | sort -u
+    */
+  case effBeginSetProgram:
+  case effCanBeAutomated:
+  case effCanDo:
+  case effClose:
+  case effConnectInput:
+  case effConnectOutput:
+  case effEditClose:
+  case effEditDraw:
+  case effEditGetRect:
+  case effEditIdle:
+  case effEditMouse:
+  case effEditOpen:
+  case effEditSleep:
+  case effEditTop:
+  case effEndSetProgram:
+  case effGetChunk:
+  case effGetCurrentMidiProgram:
+  case effGetEffectName:
+  case effGetInputProperties:
+  case effGetMidiKeyName:
+  case effGetNumMidiInputChannels:
+  case effGetNumMidiOutputChannels:
+  case effGetOutputProperties:
+  case effGetParamDisplay:
+  case effGetParamLabel:
+  case effGetParamName:
+  case effGetPlugCategory:
+  case effGetProductString:
+  case effGetProgram:
+  case effGetProgramName:
+  case effGetProgramNameIndexed:
+  case effGetSpeakerArrangement:
+  case effGetTailSize:
+  case effGetVendorString:
+  case effGetVendorVersion:
+  case effGetVstVersion:
+  case effIdentify:
+  case effIdle:
+  case effKeysRequired:
+  case effMainsChanged:
+  case effOpen:
+  case effProcessEvents:
+  case effSetBlockSize:
+  case effSetBypass:
+  case effSetChunk:
+  case effSetProcessPrecision:
+  case effSetProgram:
+  case effSetProgramName:
+  case effSetSampleRate:
+  case effSetSpeakerArrangement:
+  case effSetTotalSampleToProcess:
+  case effShellGetNextPlugin:
+  case effStartProcess:
+  case effStopProcess:
+  case effString2Parameter:
+  case effVendorSpecific:
+    return 1;
   }
   return 0;
 }
@@ -265,80 +309,89 @@ static int hostKnown(t_fstPtrInt opcode) {
   if(opcode>=100000)
     return 0;
   switch(opcode) {
-    case audioMasterAutomate:
-    case audioMasterVersion:
-    case audioMasterCurrentId:
-    case audioMasterWantMidi:
-    case audioMasterGetTime:
-    case audioMasterProcessEvents:
-    case audioMasterTempoAt:
-    case audioMasterGetSampleRate:
-    case audioMasterSizeWindow:
-    case audioMasterGetBlockSize:
-    case audioMasterGetCurrentProcessLevel:
-    case audioMasterGetVendorString:
-    case audioMasterGetProductString:
-    case audioMasterGetVendorVersion:
-    case audioMasterCanDo:
-    case audioMasterBeginEdit:
-    case audioMasterEndEdit:
-    case audioMasterCloseWindow:
-    case audioMasterOpenWindow:
-    case audioMasterSetIcon:
-    case audioMasterUpdateDisplay:
-    case audioMasterGetParameterQuantization:
-    case audioMasterGetNumAutomatableParameters:
-    case audioMasterGetAutomationState:
-    case audioMasterGetInputLatency:
-    case audioMasterGetOutputLatency:
-    case audioMasterGetDirectory:
-    case audioMasterGetLanguage:
-    case audioMasterGetOutputSpeakerArrangement:
-    case audioMasterOfflineGetCurrentMetaPass:
-    case audioMasterOfflineGetCurrentPass:
-    case audioMasterOfflineRead:
-    case audioMasterOfflineStart:
-    case audioMasterOfflineWrite:
-    case audioMasterGetNextPlug:
-    case audioMasterGetPreviousPlug:
-    case audioMasterIdle:
-    case audioMasterNeedIdle:
-    case audioMasterIOChanged:
-    case audioMasterPinConnected:
-    case audioMasterSetOutputSampleRate:
-    case audioMasterSetTime:
-    case audioMasterWillReplaceOrAccumulate:
-    case audioMasterVendorSpecific:
-      return 1;
   default: break;
+    /*
+      grep "^[[:space:]]*FST_HOST_OPCODE" fst/fst.h | sed -e 's|,.*||' -e 's|).*||' -e 's|$|:|' -e 's|.*(|  case audioMaster|' | sort -u
+    */
+  case audioMasterAutomate:
+  case audioMasterBeginEdit:
+  case audioMasterCanDo:
+  case audioMasterCloseFileSelector:
+  case audioMasterCloseWindow:
+  case audioMasterCurrentId:
+  case audioMasterEditFile:
+  case audioMasterEndEdit:
+  case audioMasterGetAutomationState:
+  case audioMasterGetBlockSize:
+  case audioMasterGetChunkFile:
+  case audioMasterGetCurrentProcessLevel:
+  case audioMasterGetDirectory:
+  case audioMasterGetInputLatency:
+  case audioMasterGetInputSpeakerArrangement:
+  case audioMasterGetLanguage:
+  case audioMasterGetNextPlug:
+  case audioMasterGetNumAutomatableParameters:
+  case audioMasterGetOutputLatency:
+  case audioMasterGetOutputSpeakerArrangement:
+  case audioMasterGetParameterQuantization:
+  case audioMasterGetPreviousPlug:
+  case audioMasterGetProductString:
+  case audioMasterGetSampleRate:
+  case audioMasterGetTime:
+  case audioMasterGetVendorString:
+  case audioMasterGetVendorVersion:
+  case audioMasterIOChanged:
+  case audioMasterIdle:
+  case audioMasterNeedIdle:
+  case audioMasterOfflineGetCurrentMetaPass:
+  case audioMasterOfflineGetCurrentPass:
+  case audioMasterOfflineRead:
+  case audioMasterOfflineStart:
+  case audioMasterOfflineWrite:
+  case audioMasterOpenFileSelector:
+  case audioMasterOpenWindow:
+  case audioMasterPinConnected:
+  case audioMasterProcessEvents:
+  case audioMasterSetIcon:
+  case audioMasterSetOutputSampleRate:
+  case audioMasterSetTime:
+  case audioMasterSizeWindow:
+  case audioMasterTempoAt:
+  case audioMasterUpdateDisplay:
+  case audioMasterVendorSpecific:
+  case audioMasterVersion:
+  case audioMasterWantMidi:
+  case audioMasterWillReplaceOrAccumulate:
+    return 1;
   }
   return 0;
 }
 
-static void print_aeffect(AEffect*eff) {
-  printf("AEffect @ %p", eff);
+static void post_aeffect(AEffect*eff) {
+  post("AEffect @ %p", eff);
   if(!eff)return;
   if(eff->magic != 0x56737450)
-    printf("\n\tmagic=0x%X", eff->magic);
+    post("\tmagic=0x%X", eff->magic);
   else
-    printf("\n\tmagic=VstP");
-  printf("\n\tdispatcherCB=%p", eff->dispatcher);
-  printf("\n\tprocessCB=%p", eff->process);
-  printf("\n\tgetParameterCB=%p", eff->getParameter);
-  printf("\n\tsetParameterCB=%p", eff->setParameter);
+    post("\tmagic=VstP");
+  post("\tdispatcherCB=%p", eff->dispatcher);
+  post("\tprocessCB=%p", eff->process);
+  post("\tgetParameterCB=%p", eff->getParameter);
+  post("\tsetParameterCB=%p", eff->setParameter);
 
-  printf("\n\tnumPrograms=%d", eff->numPrograms);
-  printf("\n\tnumParams=%d", eff->numParams);
-  printf("\n\tnumInputs=%d", eff->numInputs);
-  printf("\n\tnumOutputs=%d", eff->numOutputs);
+  post("\tnumPrograms=%d", eff->numPrograms);
+  post("\tnumParams=%d", eff->numParams);
+  post("\tnumInputs=%d", eff->numInputs);
+  post("\tnumOutputs=%d", eff->numOutputs);
 
   int flags = eff->flags;
-  printf("\n\tflags="); print_binary(flags);
+  printf("FST::\tflags="); print_binary(flags); printf("\n");
+
 #define FST_UTILS__FLAG(x) if(effFlags##x) {                      \
-    if(effFlags##x & flags)printf("\n\t      %s", #x);            \
+    if(effFlags##x & flags)post("\t      %s", #x);                \
     flags &= ~effFlags##x;                                        \
   }                                                               \
-  else printf("\n\t      ???%s???", #x)
+  else post("\t      ???%s???", #x)
 
   FST_UTILS__FLAG(HasEditor);
   FST_UTILS__FLAG(IsSynth);
@@ -347,92 +400,94 @@ static void print_aeffect(AEffect*eff) {
   FST_UTILS__FLAG(NoSoundInStop);
   FST_UTILS__FLAG(ProgramChunks);
   if(flags) {
-    printf("\n\t      ");
+    printf("FST::\t      ");
     print_binary(flags);
+    printf("\n");
   }
 
-  printf("\n\tresvd1=0x%X", eff->resvd1);
-  printf("\n\tresvd2=0x%X", eff->resvd2);
-  printf("\n\tinitialDelay=%d", eff->initialDelay);
+  post("\tresvd1=0x%X", eff->resvd1);
+  post("\tresvd2=0x%X", eff->resvd2);
+  post("\tinitialDelay=%d", eff->initialDelay);
 
-  printf("\n\tuser=%p", eff->user);
-  printf("\n\tobject=%p", eff->object);
-  printf("\n\tuniqueID=%d", eff->uniqueID);
-  printf("\n\tversion=%d", eff->version);
+  post("\tuser=%p", eff->user);
+  post("\tobject=%p", eff->object);
+  post("\tuniqueID=%d", eff->uniqueID);
+  post("\tversion=%d", eff->version);
 
-  printf("\n\tprocessReplacingCB=%p", eff->processReplacing);
-  printf("\n\tprocessDoubleReplacingCB=%p", eff->processDoubleReplacing);
-  printf("\n\n");
+  post("\tprocessReplacingCB=%p", eff->processReplacing);
+  post("\tprocessDoubleReplacingCB=%p", eff->processDoubleReplacing);
+  post("");
 }
-static void print_event(VstEvent*ev, int hexdump, const char*prefix="") {
-  printf("%sVstEvent @ %p", prefix, ev);
+static void post_event(VstEvent*ev, int hexdump, const char*prefix="") {
+  printf("FST::%sVstEvent @ %p", prefix, ev);
   if(!ev) {
     printf(" [%d]\n", sizeof(VstEvent));
     return;
   }
+  printf("\n");
   if(hexdump) {
-    printf("\n");
-    print_hex(ev, ev->byteSize + 4*(kVstMidiType == ev->type));
+    post_hex(ev, ev->byteSize + 4*(kVstMidiType == ev->type));
   }
 
   if(ev->type == kVstMidiType) {
     VstMidiEvent*mev = (VstMidiEvent*)ev;
-    printf("%s [%d]\n", prefix, sizeof(VstMidiEvent));
+    post("%s [%d]", prefix, sizeof(VstMidiEvent));
 
-    printf("%s", prefix); printf("\ttype=%d\n", mev->type);
-    printf("%s", prefix); printf("\tbyteSize=%d\n\tdeltaFrames=%d\n", mev->byteSize, mev->deltaFrames);
-    printf("%s", prefix); printf("\tMIDI: %02x %02x %02x %02x\n"
+    post("\ttype=%d", mev->type);
+    post("\tbyteSize=%d", mev->byteSize);
+    post("\tdeltaFrames=%d", mev->deltaFrames);
+    post("\tMIDI: %02x %02x %02x %02x"
            , mev->midiData[0]
            , mev->midiData[1]
            , mev->midiData[2]
            , mev->midiData[3]);
-    printf("%s", prefix); printf("\tnote: length=%d\toffset=%d\tvelocity=%d\tdetune=%d\n",
+    post("\tnote: length=%d\toffset=%d\tvelocity=%d\tdetune=%d",
       mev->noteLength,
       mev->noteOffset,
       mev->noteOffVelocity,
       mev->detune);
   } else if (ev->type == kVstSysExType) {
     VstMidiSysexEvent*sev = (VstMidiSysexEvent*)ev;
-    printf("%s", prefix); printf(" [%d]\n", sizeof(VstMidiSysexEvent));
+    post(" [%d]", sizeof(VstMidiSysexEvent));
 
-    printf("%s", prefix); printf("\ttype=%d\n", sev->type);
-    printf("%s", prefix); printf("\tbyteSize=%d\n\tdeltaFrames=%d\n", sev->byteSize, sev->deltaFrames);
-    printf("%s", prefix); printf("\tSysEx %d bytes @ %p\n\t", sev->dumpBytes, sev->sysexDump);
+    post("\ttype=%d", sev->type);
+    post("\tbyteSize=%d", sev->deltaFrames);
+    post("\tdeltaFrames=%d", sev->deltaFrames);
+    post("\tSysEx %d bytes @ %p", sev->dumpBytes, sev->sysexDump);
     unsigned char*data=(unsigned char*)sev->sysexDump;
-    printf("%s", prefix);
+    printf("FST::\t%s", prefix);
     for(int i=0; i<sev->dumpBytes; i++)
       printf(" %02x", *data++);
     printf("\n");
-    printf("\tflags=%d\treserved=%lu\t%lu\n",
+    post("\tflags=%d\treserved=%lu\t%lu",
         sev->flags, sev->resvd1, sev->resvd2);
   }
 }
 
-static void print_events(VstEvents*evs, int hexdump=0, const char*prefix="") {
-  printf("%s%d VstEvents @ %p\n", prefix, evs?evs->numEvents:0, evs);
+static void post_events(VstEvents*evs, int hexdump=0, const char*prefix="") {
+  post("%s%d VstEvents @ %p", prefix, evs?evs->numEvents:0, evs);
   if(!evs)return;
   for(int i=0; i<evs->numEvents; i++) {
-    print_event(evs->events[i], hexdump, prefix);
+    post_event(evs->events[i], hexdump, prefix);
   }
 }
 
-static void print_erect(ERect*rect, const char*prefix="") {
-  printf("%sERect[%p]", prefix, rect);
+static void post_erect(ERect*rect, const char*prefix="") {
+  printf("FST::%sERect[%p]", prefix, rect);
   if(rect)
     printf(" = %d|%d - %d|%d", rect->top, rect->left, rect->bottom, rect->right);
   printf("\n");
 }
-static void print_pinproperties(VstPinProperties*vpp) {
-  printf("VstPinProperties @ %p", vpp);
+static void post_pinproperties(VstPinProperties*vpp) {
+  post("VstPinProperties @ %p", vpp);
   if(!vpp) {
-    printf("\n");
     return;
   }
-  printf("\nlabel     : '%.*s'", 64, vpp->label);
-  printf("\nlshortabel: '%.*s'",  8, vpp->shortLabel);
-  printf("\narrangtype: %d", vpp->arrangementType);
-  printf("\nflags     : %d", vpp->flags);
-  printf("\n");
+  post("\tlabel     : '%.*s'", 64, vpp->label);
+  post("\tlshortabel: '%.*s'",  8, vpp->shortLabel);
+  post("\tarrangtype: %X", vpp->arrangementType);
+  printf("FST::\tflags     : ");print_binary(vpp->flags);printf("\n");
+
 }
 
 static char*speakerArrangement2string(int type, char*output, size_t length) {
@@ -508,51 +563,48 @@ static char*speaker2string(VstSpeakerProperties*props, char*output, size_t lengt
   output[length-1]=0;
   return output;
 }
-static void print_non0bytes(void*bytes, size_t length) {
+static void post_non0bytes(void*bytes, size_t length) {
   char*data=(char*)bytes;
   for(size_t i=0; i<length; i++) {
     if(data[i]) {
-      printf("\n padding ");
-      print_hex(data, length);
+      printf("\nFST:: padding ");
+      post_hex(data, length);
       return;
     }
   }
 }
-static void print_speakerpadding(VstSpeakerProperties*props) {
+static void post_speakerpadding(VstSpeakerProperties*props) {
   /* print padding bytes if non-0 */
-  print_non0bytes(props, 80);
-  print_non0bytes(props->_padding2, sizeof(props->_padding2));
+  post_non0bytes(props, 80);
+  post_non0bytes(props->_padding2, sizeof(props->_padding2));
 }
 
-static void print_speakerarrangement(const char*name, VstSpeakerArrangement*vpp) {
+static void post_speakerarrangement(const char*name, VstSpeakerArrangement*vpp) {
   char buf[512];
-  printf("SpeakerArrangement[%s] @ %p: %s", name, vpp, (vpp?speakerArrangement2string(vpp->type, buf, 512):0));
+  post("SpeakerArrangement[%s] @ %p: '%s'", name, vpp, (vpp?speakerArrangement2string(vpp->type, buf, 512):0));
   if(!vpp) {
-    printf("\n");
     return;
   }
   for(int i=0; i < vpp->numChannels; i++) {
-    printf("\n\t#%d: %s", i, speaker2string(&(vpp->speakers[i]), buf, 512));
-    print_speakerpadding(&(vpp->speakers[i]));
+    post("\t#%d: %s", i, speaker2string(&(vpp->speakers[i]), buf, 512));
+    post_speakerpadding(&(vpp->speakers[i]));
   }
-  printf("\n");
-  //print_hex(vpp, 1024);
+  //post_hex(vpp, 1024);
 }
 
 
-static void print_timeinfo(VstTimeInfo*vti) {
-  printf("VstTimeInfo @ %p", vti);
+static void post_timeinfo(VstTimeInfo*vti) {
+  post("VstTimeInfo @ %p", vti);
   if(!vti) {
-    printf("\n");
     return;
   }
-#define FST_UTILS__VTI_g(ti, x) printf("\n\t%s: %g", #x, ti->x)
-#define FST_UTILS__VTI_d(ti, x) printf("\n\t%s: %d", #x, ti->x)
-#define FST_UTILS__VTI_x(ti, x) printf("\n\t%s: 0x%X", #x, ti->x)
+#define FST_UTILS__VTI_g(ti, x) post("\t%s: %g", #x, ti->x)
+#define FST_UTILS__VTI_d(ti, x) post("\t%s: %d", #x, ti->x)
+#define FST_UTILS__VTI_x(ti, x) post("\t%s: 0x%X", #x, ti->x)
   FST_UTILS__VTI_g(vti, samplePos);
   FST_UTILS__VTI_g(vti, sampleRate);
   FST_UTILS__VTI_g(vti, nanoSeconds);
-  printf("\t= %gsec", (vti->nanoSeconds * 0.000000001));
+  post("\t\t= %gsec", (vti->nanoSeconds * 0.000000001));
   FST_UTILS__VTI_g(vti, ppqPos);
   FST_UTILS__VTI_g(vti, tempo);
   FST_UTILS__VTI_g(vti, barStartPos);
@@ -564,7 +616,7 @@ static void print_timeinfo(VstTimeInfo*vti) {
   FST_UTILS__VTI_x(vti, flags);
 
   int flags = vti->flags;
-#define FST_UTILS__VTI_FLAGS(flags, f) do {if(flags & f)printf("\n\t\t%s", #f); flags &= ~f;} while (0)
+#define FST_UTILS__VTI_FLAGS(flags, f) do {if(flags & f)post("\t\t%s", #f); flags &= ~f;} while (0)
   FST_UTILS__VTI_FLAGS(flags, kVstTransportChanged);
   FST_UTILS__VTI_FLAGS(flags, kVstTransportPlaying);
   FST_UTILS__VTI_FLAGS(flags, kVstTransportCycleActive);
@@ -579,21 +631,20 @@ static void print_timeinfo(VstTimeInfo*vti) {
   FST_UTILS__VTI_FLAGS(flags, kVstTimeSigValid);
   FST_UTILS__VTI_FLAGS(flags, kVstSmpteValid);
   FST_UTILS__VTI_FLAGS(flags, kVstClockValid);
-  if(flags)printf("\n\t\tremainder: 0x%04X", flags);
+  if(flags)post("\t\tremainder: 0x%04X", flags);
 
   FST_UTILS__VTI_d(vti, smpteFrameRate);
   FST_UTILS__VTI_d(vti, smpteOffset);
 
   FST_UTILS__VTI_d(vti, currentBar);
   FST_UTILS__VTI_x(vti, magic);
-  printf("\n");
 }
 
 /* direction 1: incoming (pre dispatcher)
  * direction 2: outgoing (post dispatcher)
  * retval: return-value for post-dispatcher calls
  */
-static void print_effPtr(AEffect* effect,
+static void post_effPtr(AEffect* effect,
                          int opcode, int index, t_fstPtrInt ivalue, void*ptr, float fvalue,
                          int direction, t_fstPtrInt retval=0) {
   bool incoming = direction?(direction&1):true;
@@ -602,26 +653,26 @@ static void print_effPtr(AEffect* effect,
     switch(opcode) {
     default: break;
     case effCanDo:
-      printf("\tcanDo: %s?\n", (char*)ptr);
+      post("\tcanDo: '%s'?", (char*)ptr);
       break;
     case effEditOpen:
-      printf("\twindowId: %p\n", ptr);
+      post("\twindowId: %p", ptr);
       break;
     case effSetChunk:
-      printf("\tchunk: ");
-      print_hex(ptr, ivalue);
+      printf("FST::\tchunk: ");
+      post_hex(ptr, ivalue);
       break;
     case effSetSpeakerArrangement:
-      print_speakerarrangement("input>", (VstSpeakerArrangement*)ivalue);
-      print_speakerarrangement("output>", (VstSpeakerArrangement*)ptr);
+      post_speakerarrangement("input>", (VstSpeakerArrangement*)ivalue);
+      post_speakerarrangement("output>", (VstSpeakerArrangement*)ptr);
       break;
     case effProcessEvents:
-      printf("\tevents: ");
-      print_events((VstEvents*)ptr);
+      post("\tevents: ");
+      post_events((VstEvents*)ptr,0,"\t");
       break;
     case effString2Parameter:
     case effSetProgramName:
-      printf("\t'%s'\n", (char*)ptr);
+      post("\t'%s'", (char*)ptr);
       break;
     }
   }
@@ -629,8 +680,8 @@ static void print_effPtr(AEffect* effect,
     switch(opcode) {
     default: break;
     case effGetChunk:
-      printf("\tchunk: ");
-      print_hex(ptr, retval);
+      printf("FST::\tchunk: ");
+      post_hex(ptr, retval);
       break;
     case effGetParamLabel:
     case effGetParamDisplay:
@@ -639,18 +690,18 @@ static void print_effPtr(AEffect* effect,
     case effGetProgramNameIndexed:
     case effGetProgramName:
     case effGetVendorString:
-      printf("\t'%s'\n", (char*)ptr);
+      post("\t'%s'", (char*)ptr);
       break;
     case effGetSpeakerArrangement:
-      print_speakerarrangement("input<", ((VstSpeakerArrangement**)ivalue)[0]);
-      print_speakerarrangement("output<", ((VstSpeakerArrangement**)ptr)[0]);
+      post_speakerarrangement("input<", ((VstSpeakerArrangement**)ivalue)[0]);
+      post_speakerarrangement("output<", ((VstSpeakerArrangement**)ptr)[0]);
       break;
     case effGetInputProperties:
     case effGetOutputProperties:
-      print_pinproperties((VstPinProperties*)ptr);
+      post_pinproperties((VstPinProperties*)ptr);
       break;
     case effEditGetRect:
-      print_erect((ERect*)ptr, "\t");
+      post_erect((ERect*)ptr, "\t");
     }
   }
 }
@@ -659,7 +710,7 @@ static void print_effPtr(AEffect* effect,
  * direction 2: outgoing (post dispatcher)
  * retval: return-value for post-dispatcher calls
  */
-static void print_hostPtr(AEffect* effect,
+static void post_hostPtr(AEffect* effect,
                           int opcode, int index, t_fstPtrInt ivalue, void*ptr, float fvalue,
                           int direction, t_fstPtrInt retval=0) {
   bool incoming = direction?(direction|1):true;
@@ -668,11 +719,11 @@ static void print_hostPtr(AEffect* effect,
     switch(opcode) {
     default: break;
     case audioMasterProcessEvents:
-      printf("\tevents: ");
-      print_events((VstEvents*)ptr);
+      post("\tevents: ");
+      post_events((VstEvents*)ptr, 0, "\n");
       break;
     case audioMasterCanDo:
-      printf("\tcanDo: %s?\n", (char*)ptr);
+      post("\tcanDo: '%s'?", (char*)ptr);
       break;
     }
   }
@@ -680,15 +731,15 @@ static void print_hostPtr(AEffect* effect,
     switch(opcode) {
     default: break;
     case audioMasterGetTime:
-      print_timeinfo((VstTimeInfo*)retval);
+      post_timeinfo((VstTimeInfo*)retval);
       return;
     case audioMasterGetDirectory:
-      printf("\t'%s'\n", (char*)retval);
+      post("\t'%s'", (char*)retval);
       break;
       break;
     case audioMasterGetVendorString:
     case audioMasterGetProductString:
-      printf("\t'%s'\n", (char*)ptr);
+      post("\t'%s'", (char*)ptr);
       break;
     }
   }
@@ -713,22 +764,45 @@ static
 t_fstPtrInt dispatch_effect (const char*name, AEffectDispatcherProc dispatchcb,
                              AEffect* effect, int opcode, int index, t_fstPtrInt ivalue, void*ptr, float fvalue) {
   if(effect) {
+    int verbose = 1;
     char effname[64];
     snprintf(effname, 64, "%p", effect);
     const char*effectname = name?name:effname;
     char opcodestr[256];
     if(!dispatchcb)
       dispatchcb = effect->dispatcher;
-    printf("Fst::host2plugin(%s, %s, %u=0x%X, %llu=0x%X, %p, %f)\n",
-        effectname, effCode2string(opcode, opcodestr, 255),
-        index, index,
-        ivalue, ivalue,
-        ptr, fvalue);
-    print_effPtr(effect, opcode, index, ivalue, ptr, fvalue, 1);
+    switch(opcode) {
+    default:
+      break;
+    case effIdle:
+    case effEditIdle:
+    case effEditGetRect:
+      //verbose = 0;
+      break;
+    }
+    if(verbose) {
+      if (opcode == effVendorSpecific) {
+        char opcode2str[256];
+        post("host2plugin('%s', '%s', '%s', %s, %p, %f)",
+             effectname, effCode2string(opcode, opcodestr, 255),
+             effCode2string(index, opcode2str, 255),
+             int2str(ivalue),
+             ptr, fvalue);
+      } else {
+        post("host2plugin('%s', '%s', %s, %s, %p, %f)",
+             effectname, effCode2string(opcode, opcodestr, 255),
+             int2str(index),
+             int2str2(ivalue),
+             ptr, fvalue);
+      }
+      post_effPtr(effect, opcode, index, ivalue, ptr, fvalue, 1);
+    }
     fflush(stdout);
     t_fstPtrInt result = dispatchcb(effect, opcode, index, ivalue, ptr, fvalue);
-    printf("Fst::host2plugin: %lu (0x%lX)\n", result, result);
-    print_effPtr(effect, opcode, index, ivalue, ptr, fvalue, 2, result);
+    if(verbose) {
+      post("host2plugin: %lu (0x%lX)", result, result);
+      post_effPtr(effect, opcode, index, ivalue, ptr, fvalue, 2, result);
+    }
     fflush(stdout);
     return result;
   }
@@ -741,13 +815,16 @@ t_fstPtrInt dispatch_host (const char*name, AEffectDispatcherProc dispatchcb,
   snprintf(effname, 64, "%p", effect);
   const char*effectname = name?name:effname;
   char opcodestr[256];
-  printf("Fst::plugin2host(%s, %s, %u=0x%X, %llu=0x%X, %p, %f)\n",
-      effectname, hostCode2string(opcode, opcodestr, 255), index, index, ivalue, ivalue, ptr, fvalue);
-  print_hostPtr(effect, opcode, index, ivalue, ptr, fvalue, 1);
+  post("plugin2host('%s', '%s', %s, %s, %p, %f)",
+      effectname, hostCode2string(opcode, opcodestr, 255),
+       int2str(index),
+       int2str2(ivalue),
+       ptr, fvalue);
+  post_hostPtr(effect, opcode, index, ivalue, ptr, fvalue, 1);
   fflush(stdout);
   t_fstPtrInt result = dispatchcb(effect, opcode, index, ivalue, ptr, fvalue);
-  printf("Fst::plugin2host: %lu (0x%lX)\n", result, result);
-  print_hostPtr(effect, opcode, index, ivalue, ptr, fvalue, 2, result);
+  post("plugin2host: %lu (0x%lX)", result, result);
+  post_hostPtr(effect, opcode, index, ivalue, ptr, fvalue, 2, result);
 
   fflush(stdout);
   return result;
@@ -759,20 +836,20 @@ t_fstMain* fstLoadPlugin(const char* filename) {
   t_fstMain*vstfun = 0;
 #ifdef _WIN32
   HINSTANCE handle = LoadLibrary(filename);
-  printf("loading %s as %p\n", filename, handle);
-  if(!handle){printf("\tfailed!\n"); return 0; }
+  post("loading %s as %p", filename, handle);
+  if(!handle){post("\tfailed!"); return 0; }
   if(!vstfun)vstfun=(t_fstMain*)GetProcAddress(handle, "VSTPluginMain");
   if(!vstfun)vstfun=(t_fstMain*)GetProcAddress(handle, "main");
   if(!vstfun)FreeLibrary(handle);
 #else
   void*handle = dlopen(filename, RTLD_NOW | RTLD_GLOBAL);
-  printf("loading %s as %p\n", filename, handle);
-  if(!handle){printf("\t%s\n", dlerror()); return 0; }
+  post("loading %s as %p", filename, handle);
+  if(!handle){post("\t%s", dlerror()); return 0; }
   if(!vstfun)vstfun=(t_fstMain*)dlsym(handle, "VSTPluginMain");
   if(!vstfun)vstfun=(t_fstMain*)dlsym(handle, "main");
   if(!vstfun)dlclose(handle);
 #endif
-  printf("loaded '%s' @ %p: %p\n", filename, handle, vstfun);
+  post("loaded '%s' @ %p: %p", filename, handle, vstfun);
   fflush(stdout);
   fstpause(1.);
   return vstfun;
